﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows;
using AmazonGlacierGUI.Core;
using AmazonGlacierGUI.Core.Helpers.AmazonGlacier;
using AmazonGlacierGUI.Core.Infrastructure;
using AmazonGlacierGUI.Core.Model;
using AmazonGlacierGUI.UI.Infrastructure;
using AmazonGlacierGUI.UI.Infrastructure.MVVMFramework;
using GalaSoft.MvvmLight.Command;
using Raven.Database.Tasks;
using System.Threading.Tasks;
using Task = System.Threading.Tasks.Task;

namespace AmazonGlacierGUI.UI.ViewModels
{
    //20-Sep-2012 Priyan R
    public class DownloadArchiveFromUploadHistoryViewModel : AppViewModelBase
    {
        private AmazonGlacierHelper _helper;            
        public DownloadArchiveFromUploadHistoryViewModel()
        {
            _helper=new AmazonGlacierHelper();
            _helper.AccessKey = AppContext.Current.Settings.AccessKey;
            _helper.SecretAccessKey = AppContext.Current.Settings.SecretAccessKey;            
            WireCommands();
            AddValidation(() => SaveToPath,
                          () =>
                          {
                              if (!Directory.Exists(SaveToPath))
                              {
                                  return new ValidationResultData("Path not exist");
                              }
                              return ValidationResultData.Success;
                          });
        }
        
        public override void OnInit()
        {
    
        }
        private void OnDownloadCommand()
        {            
            if(!ValidateAll()) return;
            foreach (var archiveDetail in ArchiveDetails.Where(p=>p.IsSelected))
            {
                if(!archiveDetail.ValidateAll()) return;
            }
            foreach (var archiveDetail in ArchiveDetails.Where(p=>p.IsSelected))
            {
                var vm = new JobViewModel();
                vm.JobType=JobTypeEnum.Download;
                vm.DownloadArchiveId = archiveDetail.ArchiveId;
                vm.DownloadArchiveToPath = SaveToPath;
                vm.DownlodArchiveFileName = archiveDetail.SaveAsFileName;
                vm.VaultName = VaultName;
                App.JobsVM.AddJob(vm);
            }
            Close();
        }
        private void OnBrowseVaultCommand()
        {
            if (!ValidateProperty(() => Region)) return;
            var vm = new VaultBrowserViewModel(Region.Value);
            UIHelper.ShowVaultBrowser(vm);
            if (vm.SelectedVault != null)
            {
                VaultName = vm.SelectedVault.VaultName;
                Region = vm.Region;
            }

        }
        private void OnBrowseFolderCommand()
        {
            UIHelper.ShowSelectFolderDialog(
                (path) =>
                {
                    SaveToPath = path;
                }, () => { });
        }
        void LoadHistory()
        {
            if (Region == null || VaultName == null)
            {
                ArchiveDetails = null;
                return;
            }
            IsLoading = true;
            Task.Factory.StartNew(
                () =>
                {
                    var results = AppContext.Current.GetServiceFactory().AppService.GetUploadHistory(Region.Value,
                                                                                                     VaultName);
                    ExecuteOnUI(
                        () =>
                        {
                            _archiveDetails = new List<ArchiveDetailsVM>();
                            foreach (var result in results)
                            {
                                _archiveDetails.Add(new ArchiveDetailsVM(result));
                            }
                            RaisePropertyChanged(() => ArchiveDetails);
                        });
                }).
                ContinueWith(
                (t)=>
                {
                    ExecuteOnUI(() =>
                                {
                                    IsLoading = false;
                                    if (t.Exception != null)
                                    {
                                        UIHelper.ShowError(AppUtility.GetExceptionMessage(t.Exception), () => { });
                                    }                                    
                                });
                });
        }
        private void WireCommands()
        {
            DownloadCommand = new RelayCommand(OnDownloadCommand);
            BrowseVaultCommand = new RelayCommand(OnBrowseVaultCommand);
            BrowseFolderCommand = new RelayCommand(OnBrowseFolderCommand);
        }

     
        private List<ArchiveDetailsVM> _archiveDetails;
        public List<ArchiveDetailsVM> ArchiveDetails
        {
            get { return _archiveDetails; }
            set
            {
                if (_archiveDetails != value)
                {
                    _archiveDetails = value;                    
                    RaisePropertyChanged(() => ArchiveDetails);
                }
            }
        }

        private string _saveToPath;
        public string SaveToPath
        {
            get { return _saveToPath; }
            set
            {
                if (_saveToPath != value)
                {
                    _saveToPath = value;                    
                    RaisePropertyChanged(() => SaveToPath);
                }
            }
        }
        private string _vaultName;
        [Required]
        public string VaultName
        {
            get { return _vaultName; }
            set
            {
                if (_vaultName != value)
                {
                    _vaultName = value;
                    RaisePropertyChanged(() => VaultName);
                    LoadHistory();
                }
            }
        }
        private AmazonRegionEnum? _region;
        [Required]
        public AmazonRegionEnum? Region
        {
            get { return _region; }
            set
            {
                if (_region != value)
                {
                    _region = value;
                    RaisePropertyChanged(() => Region);
                    if (value != null)
                    {
                        _helper.Region = value.Value;
                    }
                    VaultName = null;
                    LoadHistory();
                }
            }
        }
        private bool _isLoading;
        public bool IsLoading
        {
            get { return _isLoading; }
            set
            {
                if (_isLoading != value)
                {
                    _isLoading = value;
                    RaisePropertyChanged(() => IsLoading);
                }
            }
        }
        public RelayCommand DownloadCommand { get; set; }
        public RelayCommand BrowseFolderCommand { get; set; }        
        public RelayCommand BrowseVaultCommand { get; set; }
        public class ArchiveDetailsVM:AppViewModelBase
        {
            public ArchiveDetailsVM()
            {
                AddValidation(() => SaveAsFileName,
                              () =>
                              {
                                  if(string.IsNullOrWhiteSpace(SaveAsFileName)==false&&AppUtility.IsValidFileName(SaveAsFileName)==false)
                                  {
                                      return new ValidationResultData("Invalid file name");
                                  }
                                  return ValidationResultData.Success;
                              });
            }
            public ArchiveDetailsVM(UploadHistory history):this()
            {
                ArchiveId = history.ArchiveId;
                SaveAsFileName=FileName = Path.GetFileName(history.FilePath);
                Size = history.FileSize;
                UploadedOn = history.Date;
                Description = history.Description;
                EnableValidtion = true;
            }
            private string _archiveId;
            public string ArchiveId
            {
                get { return _archiveId; }
                set
                {
                    if (_archiveId != value)
                    {
                        _archiveId = value;             
                        RaisePropertyChanged(() => ArchiveId);
                    }
                }
            }

            private string _fileName;
            public string FileName
            {
                get { return _fileName; }
                set
                {
                    if (_fileName != value)
                    {
                        _fileName = value;
                        RaisePropertyChanged(() => FileName);
                    }
                }
            }
            private string _saveAsFileName;
            [Required]
            public string SaveAsFileName
            {
                get { return _saveAsFileName; }
                set
                {
                    if (_saveAsFileName != value)
                    {
                        _saveAsFileName = value;                        
                        RaisePropertyChanged(() => SaveAsFileName);
                    }
                }
            }
            
            private bool _isSelected;
            public bool IsSelected
            {
                get { return _isSelected; }
                set
                {
                    if (_isSelected != value)
                    {
                        _isSelected = value;                        
                        RaisePropertyChanged(() => IsSelected);
                    }
                }
            }
            private DateTime _uploadedOn;
            public DateTime UploadedOn
            {
                get { return _uploadedOn; }
                set
                {
                    if (_uploadedOn != value)
                    {
                        _uploadedOn = value;             
                        RaisePropertyChanged(() => UploadedOn);
                    }
                }
            }
            private long _size;
            public long Size
            {
                get { return _size; }
                set
                {
                    if (_size != value)
                    {
                        _size = value;                        
                        RaisePropertyChanged(() => Size);
                    }
                }
            }

            private string _description;
            public string Description
            {
                get { return _description; }
                set
                {
                    if (_description != value)
                    {
                        _description = value;
                        RaisePropertyChanged(() => Description);
                    }
                }
            }
        }
    }
}